"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var TreeNode = /** @class */ (function () {
    function TreeNode(_a) {
        var start = _a.start, end = _a.end, index = _a.index;
        this.start = start;
        this.end = end;
        this._index = index;
    }
    Object.defineProperty(TreeNode.prototype, "index", {
        get: function () {
            return this._index;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(TreeNode.prototype, "left", {
        get: function () {
            return this._left;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(TreeNode.prototype, "right", {
        get: function () {
            return this._right;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(TreeNode.prototype, "leftHeight", {
        get: function () {
            return this.left ? this.left.height : 0;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(TreeNode.prototype, "rightHeight", {
        get: function () {
            return this.right ? this.right.height : 0;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(TreeNode.prototype, "height", {
        get: function () {
            return Math.max(this.leftHeight, this.rightHeight) + 1;
        },
        enumerable: true,
        configurable: true
    });
    TreeNode.prototype.rotateRight = function () {
        if (this.left) {
            var head = this.left;
            this._left = this.left.right;
            head._right = this;
            return head;
        }
        return this;
    };
    TreeNode.prototype.rotateLeft = function () {
        if (this.right) {
            var head = this.right;
            this._right = this.right.left;
            head._left = this;
            return head;
        }
        return this;
    };
    TreeNode.prototype.balanceTree = function () {
        if (Math.abs(this.leftHeight - this.rightHeight) > 1) {
            return this.leftHeight > this.rightHeight ? this.rotateRight() : this.rotateLeft();
        }
        return this;
    };
    TreeNode.prototype.add = function (_a) {
        var start = _a.start, end = _a.end, index = _a.index;
        if (this.start >= end) {
            this._left = this.left ? this.left.add({ start: start, end: end, index: index }) : new TreeNode({ start: start, end: end, index: index });
        }
        else if (this.end <= start) {
            this._right = this.right ? this.right.add({ start: start, end: end, index: index }) : new TreeNode({ start: start, end: end, index: index });
        }
        else if (this.start !== start || this.end !== end || this._index !== index) {
            throw new Error('Collision');
        }
        return this.balanceTree();
    };
    TreeNode.prototype.get = function (pixel) {
        if (this.start > pixel) {
            return this.left ? this.left.get(pixel) : -1;
        }
        else if (this.end <= pixel) {
            return this.right ? this.right.get(pixel) : -1;
        }
        else {
            return this.index;
        }
    };
    return TreeNode;
}());
exports.TreeNode = TreeNode;
var IndexCache = /** @class */ (function () {
    function IndexCache() {
    }
    IndexCache.prototype.set = function (params) {
        if (!this.treeNode) {
            this.treeNode = new TreeNode(params);
        }
        else {
            this.treeNode = this.treeNode.add(params);
        }
    };
    IndexCache.prototype.get = function (pixel) {
        return this.treeNode ? this.treeNode.get(pixel) : -1;
    };
    IndexCache.prototype.has = function (pixel) {
        return this.get(pixel) !== -1;
    };
    return IndexCache;
}());
exports.default = IndexCache;
